"""App Configuration"""

# Django
from django.apps import AppConfig
from django.db.models.signals import post_migrate
from anomalytracker import __version__
from django.core.management import call_command


def clean_permissions(sender, **kwargs):
    call_command("clean_permissions", "anomalytracker")


class AnomalytrackerConfig(AppConfig):
    name = "anomalytracker"

    def ready(self):
        post_migrate.connect(clean_permissions, sender=self)



class anomalytrackerConfig(AppConfig):
    """App Config"""

    name = "anomalytracker"
    label = "anomalytracker"
    verbose_name = f"anomalytracker App v{__version__}"

    def ready(self):
        post_migrate.connect(clean_permissions, sender=self)



# Django
from django.db import models
from django.utils import timezone
from eveuniverse.models import EveSolarSystem
from datetime import timedelta
from django.contrib.auth.models import Group
from .utils import send_discord_respawn_alert
import math

class General(models.Model):
    """Meta model for app permissions"""

    class Meta:
        """Meta definitions"""

        managed = False
        default_permissions = ()
        permissions = [
            ("basic_access", "Can access this app"),
            ("manage_anoms", "Can manage anomalies"),
        ]
        default_permissions = ()

class AnomalyTierSettings(models.Model):
    tier = models.CharField(
        max_length=10,
        choices=[("t1", "Tier 1"), ("t2", "Tier 2"), ("t3", "Tier 3"), ("ice","Ice")],
        unique=True
    )
    respawn_time = models.IntegerField(default=60)

    class Meta:
        default_permissions = ()

    def __str__(self):
        return f"{self.get_tier_display()} ({self.respawn_time} min)"


class Anomalies(models.Model):
    class Meta:
        default_permissions = ()

    is_up = models.BooleanField(default=True)
    roll_started_at = models.DateTimeField(null=True, blank=True)
    next_respown = models.BooleanField(default=False)
    all_respown = models.BooleanField(default=False)

    anom_system = models.ForeignKey(
        EveSolarSystem,
        on_delete=models.CASCADE,
        help_text="system of the anomally",
    )

    anom_tier = models.ForeignKey(
        AnomalyTierSettings,
        on_delete=models.CASCADE,
        related_name="anomalies"
    )

    def save(self, *args, **kwargs):
        if self.ore == "ice":
            self.anom_tier, _ = AnomalyTierSettings.objects.get_or_create(tier="ice")
        super().save(*args, **kwargs)




    ore = models.CharField(
        max_length=15,
        choices=[("Veldspar","Tritanium"), ("Mordunium","Pyerite"), 
                 ("Kylixium","Mexallon"), ("Griemeer","Isogen"), 
                 ("Nocxite","Nocxium"), ("Hezorime","Zydrine"), 
                 ("Neganite","Megacyte"), ("Ice", "Ice"),],
        default="",
    )

    def get_respawn_duration_minutes(self):
        return self.anom_tier.respawn_time

    @property
    def is_timer_active(self):
        """Check if timer is still running. Auto-flip is_up when done."""
        if self.is_up or not self.roll_started_at:
            return False

        duration = timedelta(minutes=self.get_respawn_duration_minutes())
        end_time = self.roll_started_at + duration

        if timezone.now() >= end_time:
            # ⏰ Timer done → mark anomaly up
            self.is_up = True
            self.roll_started_at = None
            self.save(update_fields=["is_up", "roll_started_at"])

            send_discord_respawn_alert(self)
            return False

        return True

    @property
    def time_left(self):
        """Return time left (in seconds) before respawn."""
        if not self.is_timer_active:
            return 0

        duration = timedelta(minutes=self.get_respawn_duration_minutes())
        end_time = self.roll_started_at + duration
        remaining = end_time - timezone.now()
        return max(0, remaining.total_seconds())
    
    @property
    def time_left_hhmm(self):
        """Format time left as H:MM or MMm"""
        seconds = self.time_left
        hours = int(seconds // 3600)
        minutes = math.ceil((seconds % 3600) / 60)
        if hours:
            return f"{hours}h{minutes:02d}m"
        return f"{minutes} m"

    def __str__(self):
        return f"Anomalies: {self.anom_system}"
        
class Discord(models.Model):

    webhook_active = models.BooleanField(
        default=False
    )

    name = models.CharField(
        max_length=150
        )

    webhook_url = models.CharField(
        max_length=500
        )
    
    ping_target = models.CharField(
        max_length=100,
        blank=True,
        help_text="Select @everyone, @here, or a group name from AllianceAuth."
    )

    class Meta:
        default_permissions = ()

    




# Generated by Django 4.2.23 on 2025-08-07 13:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('anomalytracker', '0002_default_config'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='anomalies',
            name='unique_anom_per_system',
        ),
    ]



"""App URLs"""

# Django
from django.urls import path

# AA anomalytracker App
from anomalytracker import views

app_name: str = "anomalytracker"

urlpatterns = [
    path("", views.index, name="index"),
    path("manage/", views.manageanoms, name="manageanoms"),
    path("roll/<int:anom_id>/", views.roll_anom, name="roll"),
    path("unroll/<int:anom_id>/", views.unroll_anom, name="unroll"),
    path("", views.manage_anomalies, name="manage"),
    path("add/", views.add_anomaly, name="add"),
    path("edit/<int:pk>/", views.edit_anomaly, name="edit"),
    path("delete/<int:pk>/", views.delete_anomaly, name="delete"),    
]



"""Admin models"""

# Django
from django.contrib import admin, messages  # noqa: F401
from django.db import models
from django import forms
from django.contrib.auth.models import Group
from .models import Anomalies, AnomalyTierSettings, Discord
from .utils import send_discord_respawn_alert

# Register your models here.

@admin.action(description="Send Test Ping (with first anomaly)")
def test_ping(modeladmin, request, queryset):
    anomaly = Anomalies.objects.first()
    if not anomaly:
        messages.error(request, "No anomalies found to use for test.")
        return

    sent_count = 0
    for webhook in queryset:
        if webhook.webhook_active:
            send_discord_respawn_alert(anomaly, discord=webhook)
            sent_count += 1

    messages.success(request, f"✅ Sent test ping to {sent_count} webhook(s).")

@admin.register(Anomalies)
class AnomaliesAdmin(admin.ModelAdmin):
    list_display = ("ore", "anom_tier", "anom_system", "is_up")


@admin.register(AnomalyTierSettings)
class AnomalyTierSettingsAdmin(admin.ModelAdmin):
    list_display = ['tier', 'respawn_time']

class DiscordMessage(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Build combined choices
        base_choices = [
            ("", "None"),
            ("@everyone", "@everyone"),
            ("@here", "@here"),
        ]
        group_choices = [('@' + g.name, '@' + g.name) for g in Group.objects.all()]
        full_choices = base_choices + group_choices

        for field in [
            "ping_target",
        ]:
            self.fields[field] = forms.ChoiceField(choices=full_choices, required=False)

@admin.register(Discord)
class DiscordAdmin(admin.ModelAdmin):
    form=DiscordMessage
    list_display = ("name", "webhook_active")
    list_editable = ("webhook_active",)
    search_fields = ("name", "webhook_url")
    list_filter = ("webhook_active",)
    actions = [test_ping]
   



# anomalytracker/utils.py
import requests
from allianceauth.services.modules.discord.models import DiscordUser
from django.contrib.auth.models import Group

def resolve_ping_target(ping_value):
    if ping_value in ["@everyone", "@here", ""]:
        return ping_value
    elif ping_value.startswith("@"):
        group_name = ping_value[1:]
        try:
            group = Group.objects.get(name=group_name)
        except Group.DoesNotExist:
            return f"@{group_name}"

        try:
            discord_group_info = DiscordUser.objects.group_to_role(group=group)
            print(f"Discord group info: {discord_group_info}")
            print(f"Role ID: {discord_group_info.get('id')}")
        except HTTPError:
            # Discord service might be down or misconfigured; fallback to plain mention
            return f"@{group_name}"

        if discord_group_info and "id" in discord_group_info:
            return f"<@&{discord_group_info['id']}>"
        else:
            return f"@{group_name}"
        

    return ""

def send_discord_respawn_alert(anomaly, discord=None):
    from .models import Discord

    if not discord:
        # Send to all active webhooks
        webhooks = Discord.objects.filter(webhook_active=True)
    else:
        webhooks = [discord]

    for webhook in webhooks:
        ping = resolve_ping_target(webhook.ping_target)

        content = (
            f"{ping}\n"
            f"# 🌌 **Anomaly Respawned!**\n"
            f"## 📍 System: **{anomaly.anom_system.name}**\n"   
            f"## ⛏️ Ore: **{anomaly.get_ore_display()}**"
            f" **{anomaly.anom_tier.get_tier_display()}**"
        )

        payload = {
            "username": "Anomaly Tracker",
            "avatar_url": "https://img.favpng.com/11/13/5/asteroid-icon-png-favpng-UXJ6DtjWqEmmH162NiEfBJWLG.jpg",
            "content": content
            }

        try:
            response = requests.post(webhook.webhook_url, json=payload)
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            print(f"Failed to send webhook to {webhook.webhook_url}: {e}")



from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.apps import apps

def run_clean_permissions(app_label: str):
    try:
        app_config = apps.get_app_config(app_label)
    except LookupError:
        print(f"No app found with label '{app_label}'")
        return

    removed_count = 0

    for model in app_config.get_models():
        ct = ContentType.objects.get_for_model(model)
        model_name = model.__name__.lower()
        for codename_prefix in ['add', 'change', 'delete', 'view']:
            codename = f"{codename_prefix}_{model_name}"
            deleted, _ = Permission.objects.filter(content_type=ct, codename=codename).delete()
            if deleted:
                print(f"Removed permission: {codename}")
                removed_count += 1

    if removed_count:
        print(f"✅ Removed {removed_count} default permissions from app '{app_label}'.")
    else:
        print(f"ℹ️ No default permissions found to remove in app '{app_label}'.")



"""App Tasks"""

# Standard Library
import logging

# Third Party
from celery import shared_task

logger = logging.getLogger(__name__)

# Create your tasks here


# anomalytracker Task
@shared_task
def anomalytracker_task():
    """anomalytracker Task"""

    pass



from django.db import migrations

def create_default_tiers(apps, schema_editor):
    AnomalyTierSettings = apps.get_model('anomalytracker', 'AnomalyTierSettings')
    default_tiers = [
        {'tier': 't1', 'respawn_time': 30},
        {'tier': 't2', 'respawn_time': 60},
        {'tier': 't3', 'respawn_time': 90},
        {'tier': 'ice', 'respawn_time': 120},
    ]
    for tier_data in default_tiers:
        AnomalyTierSettings.objects.update_or_create(tier=tier_data['tier'], defaults=tier_data)

def delete_default_tiers(apps, schema_editor):
    AnomalyTierSettings = apps.get_model('anomalytracker', 'AnomalyTierSettings')
    AnomalyTierSettings.objects.filter(tier__in=['t1', 't2', 't3', 'ice']).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('anomalytracker', '0001_initial'),
    ]

    operations = [
         migrations.RunPython(create_default_tiers, reverse_code=delete_default_tiers)

    ]



# Generated by Django 4.2.23 on 2025-08-07 11:50

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('eveuniverse', '0011_extend_industry_activites'),
    ]

    operations = [
        migrations.CreateModel(
            name='General',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'permissions': [('basic_access', 'Can access this app'), ('manage_anoms', 'Can manage anomalies')],
                'managed': False,
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='AnomalyTierSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('tier', models.CharField(choices=[('t1', 'Tier 1'), ('t2', 'Tier 2'), ('t3', 'Tier 3'), ('ice', 'Ice')], max_length=10, unique=True)),
                ('respawn_time', models.IntegerField(default=60)),
            ],
            options={
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='Discord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('webhook_active', models.BooleanField(default=False)),
                ('name', models.CharField(max_length=150)),
                ('webhook_url', models.CharField(max_length=500)),
                ('ping_target', models.CharField(blank=True, help_text='Select @everyone, @here, or a group name from AllianceAuth.', max_length=100)),
            ],
            options={
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='Anomalies',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_up', models.BooleanField(default=True)),
                ('roll_started_at', models.DateTimeField(blank=True, null=True)),
                ('next_respown', models.BooleanField(default=False)),
                ('all_respown', models.BooleanField(default=False)),
                ('ore', models.CharField(choices=[('Veldspar', 'Tritanium'), ('Mordunium', 'Pyerite'), ('Kylixium', 'Mexallon'), ('Griemeer', 'Isogen'), ('Nocxite', 'Nocxium'), ('Hezorime', 'Zydrine'), ('Neganite', 'Megacyte'), ('Ice', 'Ice')], default='', max_length=15)),
                ('anom_system', models.ForeignKey(help_text='system of the anomally', on_delete=django.db.models.deletion.CASCADE, to='eveuniverse.evesolarsystem')),
                ('anom_tier', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='anomalies', to='anomalytracker.anomalytiersettings')),
            ],
            options={
                'default_permissions': (),
            },
        ),
        migrations.AddConstraint(
            model_name='anomalies',
            constraint=models.UniqueConstraint(fields=('anom_system',), name='unique_anom_per_system'),
        ),
    ]



"""App Views"""

# Django
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import render, get_object_or_404, redirect
from django.core.handlers.wsgi import WSGIRequest
from django.http import HttpResponse
from django.contrib import messages
from django.shortcuts import render
from django.views.decorators.http import require_POST
from .models import Anomalies
from .forms import AnomalyForm
from django.utils import timezone



def manage_anomalies(request):
    anomalies = Anomalies.objects.all()
    return render(request, "anomalytracker/manageanoms.html", {"anomalies": anomalies})

def add_anomaly(request):
    if request.method == "POST":
        form = AnomalyForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(request, "Anomaly added.")
            if "add_another" in request.POST:
                return redirect("anomalytracker:add")
            return redirect("anomalytracker:manage")
    else:
        form = AnomalyForm()
    return render(request, "anomalytracker/anomaly_form.html", {"form": form, "mode": "Add"})

def edit_anomaly(request, pk):
    anomaly = get_object_or_404(Anomalies, pk=pk)
    if request.method == "POST":
        form = AnomalyForm(request.POST, instance=anomaly)
        if form.is_valid():
            form.save()
            messages.success(request, "Anomaly updated.")
            return redirect("anomalytracker:manage")
    else:
        form = AnomalyForm(instance=anomaly)
    return render(request, "anomalytracker/anomaly_form.html", {"form": form, "mode": "Edit"})

def delete_anomaly(request, pk):
    anomaly = get_object_or_404(Anomalies, pk=pk)
    anomaly.delete()
    messages.success(request, "Anomaly deleted.")
    return redirect("anomalytracker:manage")


@login_required
@permission_required("anomalytracker.basic_access")
def index(request: WSGIRequest) -> HttpResponse:
    anomalies = Anomalies.objects.exclude(ore="ice")
    ice_anomalies = Anomalies.objects.filter(ore="ice")

    context = {
        "anomalies": anomalies,
        "ice_anomalies": ice_anomalies,
    }

    return render(request, "anomalytracker/index.html", context)


@login_required
@permission_required("anomalytracker.basic_access")
def manageanoms(request: WSGIRequest) -> HttpResponse:
    anomalies = Anomalies.objects.all()
    context = {
        "anomalies": anomalies,
        "text": "anom"
    }

    return render(request, "anomalytracker/manageanoms.html", context)

@require_POST
@login_required
@permission_required("anomalytracker.basic_access")
def roll_anom(request, anom_id):
    anom = get_object_or_404(Anomalies, pk=anom_id)
    anom.is_up = False
    anom.roll_started_at = timezone.now()
    anom.save()
    return redirect("anomalytracker:index")


@require_POST
@login_required
@permission_required("anomalytracker.basic_access")
def unroll_anom(request, anom_id):
    anom = get_object_or_404(Anomalies, pk=anom_id)
    anom.is_up = True
    anom.roll_started_at = None
    anom.save()
    return redirect("anomalytracker:index")






from django import forms
from .models import Anomalies
from django.utils.translation import gettext_lazy as _
from eveuniverse.models import EveSolarSystem


class AnomalyForm(forms.ModelForm):
    class Meta:
        model = Anomalies
        fields = ["anom_system", "anom_tier", "ore"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Ustawienia pola systemu
        self.fields["anom_system"].widget.attrs.update({
            "class": "form-select select-search"
        })
        self.fields["anom_system"].queryset = EveSolarSystem.objects.all().order_by("name")

        # Dodaj Select2 do pozostałych pól
        self.fields["anom_tier"].widget.attrs.update({
            "class": "form-select"
        })
        self.fields["ore"].widget.attrs.update({
            "class": "form-select"
        })

    def clean(self):
        cleaned_data = super().clean()
        system = cleaned_data.get("anom_system")
        ore = cleaned_data.get("ore")

        if not self.instance.pk:
            if Anomalies.objects.filter(anom_system=system, ore=ore).exists():
                raise forms.ValidationError(
                    _("An anomaly with this ore already exists in the selected system.")
                )

        return cleaned_data
